{*************************************************************}
{            Blind Guardian Component for Delphi 16/32        }
{ Version:   1.0                                              }
{ Author:    Aleksey Kuznetsov, Kiev, Ukraine                 }
{              (Xacker), ,          }
{ E-Mail:    xacker@phreaker.net                              }
{ Home Page: http://www.angen.net/~xacker/                    }
{ Created:   March, 16, 1999                                  }
{ Modified:  March, 17, 1999                                  }
{ Legal:     Copyright (c) 1999 by Aleksey Xacker             }
{*************************************************************}
{   TBlindGuardian (English):                                 }
{ Protection of the Shareware programs against illegal        }
{ copying and set up the trial period.                        }
{*************************************************************}
{   TBlindGuardian (Russian):                                 }
{  Shareware          }
{  (Trial) .                                   }
{*************************************************************}
{ PROPERTIES:                                                 }
{    TrialPeriod: Word (positive value of trial period)       }
{     Registered: Boolean (registered flag)                   }
{ EVENTS:                                                     }
{    OnRegistration: Calls Registration dialog.               }
{ PROTECTED FUNCTIONS:                                        }
{    GetSettings and Modify Settings - Get and Modify the     }
{                                      protection settings    }
{ ADDITIONAL PUBLIC VARIABLES                                 }
{    StartedFirstTime: Boolean;                               }
{    DaysPassed: LongInt (number of passed days from first    }
{                                                    start)   }
{                                                             }
{ NOTE:                                                       }
{   If you are using Application.OnIdleEvent then you must    }
{   show your Registration Dialog manually.                   }
{   OnRegistration event will started only if you not used    }
{   OnIdle event.                                             }
{*************************************************************}
{                     IMPORTANT NOTE:                         }
{ This software is provided 'as-is', without any express or   }
{ implied warranty. In no event will the author be held       }
{ liable for any damages arising from the use of this         }
{ software.                                                   }
{ Permission is granted to anyone to use this software for    }
{ any purpose, including commercial applications, and to      }
{ alter it and redistribute it freely, subject to the         }
{ following restrictions:                                     }
{ 1. The origin of this software must not be misrepresented,  }
{    you must not claim that you wrote the original software. }
{    If you use this software in a product, an acknowledgment }
{    in the product documentation would be appreciated but is }
{    not required.                                            }
{ 2. Altered source versions must be plainly marked as such,  }
{    and must not be misrepresented as being the original     }
{    software.                                                }
{ 3. This notice may not be removed or altered from any       }
{    source distribution.                                     }
{*************************************************************}

unit BlindGuardian;

interface

uses
  {$IFDEF WIN32} Windows, {$ELSE} WinTypes, WinProcs, {$ENDIF}
  Messages, Classes, Controls, SysUtils, Forms;

type
  TOnRegistrationEvent = procedure(Sender: TObject; TrialPeriod, DaysPassed: Integer) of object;
  TBlindGuardian = class(TComponent)
  private
    FTrialPeriod: Word;
    FRegistered: Boolean;
    FOnRegistration: TOnRegistrationEvent;
    FirstTime: Boolean;
    DateTime: TDateTime;
    AlreadyDaysPassed: LongInt;
    OldIdle: TIdleEvent;

    procedure GuardianIdleProc(Sender: TObject; var Done: Boolean);
  protected
    procedure GetSettings; virtual;
    procedure ModifySettings; virtual;
  public
    StartedFirstTime: Boolean;
    DaysPassed: LongInt;

    constructor Create(aOwner: TComponent); override;

    procedure SetRegistered(Value: Boolean);
  published
    property TrialPeriod: Word read FTrialPeriod write FTrialPeriod;
    property Registered: Boolean read FRegistered write SetRegistered;
    property OnRegistration: TOnRegistrationEvent read FOnRegistration write FOnRegistration;
  end;

procedure Register;

implementation

constructor TBlindGuardian.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  OldIdle := Application.OnIdle;
  Application.OnIdle := GuardianIdleProc;
  FTrialPeriod := 30;
  FRegistered := False;
  AlreadyDaysPassed := 0;
  FirstTime := True;
  DateTime := Now;

  GetSettings;
end;

procedure TBlindGuardian.GuardianIdleProc(Sender: TObject; var Done: Boolean);
begin
  if FirstTime then
   begin
    if not (csDesigning in ComponentState) then
     if not FRegistered and Assigned(FOnRegistration) then
      begin
       FirstTime := False;
       FOnRegistration(Self, FTrialPeriod, DaysPassed);
      end;
    Application.OnIdle := OldIdle;
    Done := True;
   end;
end;

procedure TBlindGuardian.SetRegistered(Value: Boolean);
begin
  if FRegistered <> Value then
   begin
    FRegistered := Value;
    ModifySettings;
   end;
end;

{ It is we make that the name of a file was not
  visible in the text of -exe file }
const
  HiddenFileName: String = ''; {'WINSPOOL.CRC' xor $AA }

function GetFileName: String;
var
  pc: Array[0..$FF] of Char;
  SystemDir: String;
  i: Integer;
  St: String;
begin
  GetSystemDirectory(pc, $FF);
  SystemDir := StrPas(pc);
  St := HiddenFileName;
  for i := 1 to Length(St) do
   St[i] := char(byte(St[i]) xor $AA); { for example, $AA }
  Result := SystemDir + '\' + St;
end;

procedure TBlindGuardian.GetSettings;
var
  fs: TFileStream;
  DT: TDateTime;
begin
{ -------
  I show example only. The protected programs should be more artful
  Each way must be unique and each programmer should have own secrets
  ------- }

{ Get Registry Binding }

{ Get Windows Binding }

   try
    fs := TFileStream.Create(GetFileName, fmOpenRead);
    StartedFirstTime := False;
   except
    StartedFirstTime := True;
   end;
   if not StartedFirstTime then
    try
     fs.Read(FRegistered, SizeOf(FRegistered));
     fs.Read(DateTime, SizeOf(DateTime));
     fs.Read(AlreadyDaysPassed, SizeOf(AlreadyDaysPassed));
     fs.Free;
    except
    end;

   { Looking for passed time. }              
   if Now < DateTime then
    DT := DateTime - Now
   else
    begin
     DT := Now - DateTime;
    end;

   DaysPassed := ((Trunc((MSecsPerday * DT) / 1000) div 60) div 60) div 24 + AlreadyDaysPassed;
   if Now >= DateTime then
    begin
     AlreadyDaysPassed := DaysPassed;
     DateTime := Now;
    end;

   ModifySettings;
end;

procedure TBlindGuardian.ModifySettings;
var
  fs: TFileStream;
begin
{ -------
  I show example only. The protected programs should be more artful.
  Each way must be unique and each programmer should have own secrets.
  ------- }

{ Bind to Registry }

{ Bind to Windows }

   try
    fs := TFileStream.Create(GetFileName, fmCreate);
    fs.Write(FRegistered, SizeOf(FRegistered));
    fs.Write(DateTime, SizeOf(DateTime));
    fs.Write(AlreadyDaysPassed, SizeOf(AlreadyDaysPassed));
   except
   end;
   try
    fs.Free;
   except
   end;
end;

procedure Register;
begin
  RegisterComponents('Xacker', [TBlindGuardian]);
end;

end.
